import path from 'path';
import moduleLexer from '@ice/bundles/compiled/es-module-lexer/index.js';
import MagicString from '@ice/bundles/compiled/magic-string/index.js';

const { init, parse } = moduleLexer;

const transformImport = async (source: string, coreJsPath: string) => {
  await init;
  const [imports, exports] = parse(source);
  let s: MagicString | undefined;
  const formatPath = coreJsPath.split(path.sep).join('/');
  const str = () => s || (s = new MagicString(source));
  const isESM = exports.length > 0 || imports.some((targetImport) => {
    const importString = targetImport.n;
    return !importString.includes('core-js') && !importString.includes('@swc/helpers');
  });
  imports.forEach((targetImport) => {
    if (targetImport.n.startsWith('core-js/modules/')) {
      const replaceModule = targetImport.n.replace('core-js/',
        formatPath.endsWith('/') ? formatPath : `${formatPath}/`);
      // ESM
      if (isESM) {
        str().overwrite(
          targetImport.s,
          targetImport.e,
          replaceModule);
      } else {
        // CJS
        str().overwrite(
          targetImport.ss,
          targetImport.se,
          `require ('${replaceModule}')`);
      }
    } else if (targetImport.n.startsWith('@swc/helpers')) {
      if (!isESM) {
        // Replace @swc/helpers with cjs path.
        const importStr = source.substring(targetImport.ss, targetImport.se);
        const matchImport = importStr.match(/import\s+([\w*\s{},]*)\s+from\s+['"](.*)['"]/);
        if (matchImport) {
          const [,identifier] = matchImport;
          const replaceModule = `var ${identifier} = require('${targetImport.n.replace(/@swc\/helpers\/src\/(.*).mjs$/,
            (_, matched) => `@swc/helpers/lib/${matched}.js`)}').default`;
          str().overwrite(targetImport.ss, targetImport.se, replaceModule);
        }
      }
    }
  });
  return s ? s.toString() : source;
};

export default transformImport;
